@@ -10,18 +10,18 @@
 
 #include "kquickstyleitem_p.h"
 
-#include <qstringbuilder.h>
-#include <qpainter.h>
-#include <qpixmapcache.h>
-#include <qstyle.h>
-#include <qstylefactory.h>
+#include <QStringBuilder>
+#include <QPainter>
+#include <QPixmapCache>
+#include <QStyle>
+#include <QStyleFactory>
 #include <qstyleoption.h>
-#include <qapplication.h>
-#include <qquickwindow.h>
+#include <QApplication>
+#include <QQuickWindow>
 #include <QtQuick/qsgninepatchnode.h>
 
 #include <ksharedconfig.h>
-#include <kconfiggroup.h>
+#include <KConfigGroup>
 
 #include <Kirigami2/PlatformTheme>
 
@@ -206,8 +206,8 @@ void KQuickStyleItem::initStyleOption()
         }
         const QFont font = qApp->font("QPushButton");
         opt->fontMetrics = QFontMetrics(font);
-        QObject * menu = m_properties[QStringLiteral("menu")].value<QObject *>();
-        if (menu) {
+        bool hasMenu = m_properties[QStringLiteral("menu")].toBool();
+        if (hasMenu) {
             opt->features |= QStyleOptionButton::HasMenu;
         }
     }
@@ -295,7 +295,11 @@ void KQuickStyleItem::initStyleOption()
         QStyleOptionToolButton *opt =
                 qstyleoption_cast<QStyleOptionToolButton*>(m_styleoption);
         opt->subControls = QStyle::SC_ToolButton;
-        opt->state |= QStyle::State_AutoRaise;
+
+        if (!m_raised) {
+            opt->state |= QStyle::State_AutoRaise;
+        }
+
         opt->activeSubControls = QStyle::SC_ToolButton;
         opt->text = text();
         const QVariant icon = m_properties[QStringLiteral("icon")];
@@ -317,8 +321,7 @@ void KQuickStyleItem::initStyleOption()
         opt->iconSize = iconSize;
 
         if (m_properties.value(QStringLiteral("menu")).toBool()) {
-            opt->subControls |= QStyle::SC_ToolButtonMenu;
-            opt->features = QStyleOptionToolButton::HasMenu;
+            opt->features |= QStyleOptionToolButton::HasMenu;
         }
 
         const int toolButtonStyle = m_properties.value(QStringLiteral("toolButtonStyle")).toInt();
@@ -502,6 +505,26 @@ void KQuickStyleItem::initStyleOption()
         if (m_properties.value(QStringLiteral("partiallyChecked")).toBool())
             opt->state |= QStyle::State_NoChange;
         opt->text = text();
+
+        const QVariant icon = m_properties[QStringLiteral("icon")];
+        if (icon.canConvert<QIcon>()) {
+            opt->icon = icon.value<QIcon>();
+        } else if (icon.canConvert<QUrl>() && icon.value<QUrl>().isLocalFile()) {
+            opt->icon = QIcon(icon.value<QUrl>().toLocalFile());
+        } else if (icon.canConvert<QString>()) {
+            opt->icon = m_theme->iconFromTheme(icon.value<QString>(), m_properties[QStringLiteral("iconColor")].value<QColor>());
+        }
+        auto iconSize = QSize(m_properties[QStringLiteral("iconWidth")].toInt(), m_properties[QStringLiteral("iconHeight")].toInt());
+        if (iconSize.isEmpty()) {
+            int e = KQuickStyleItem::style()->pixelMetric(QStyle::PM_ButtonIconSize, m_styleoption, nullptr);
+            if (iconSize.width() <= 0) {
+                iconSize.setWidth(e);
+            }
+            if (iconSize.height() <= 0) {
+                iconSize.setHeight(e);
+            }
+        }
+        opt->iconSize = iconSize;
     }
         break;
     case Edit: {
@@ -680,10 +703,13 @@ void KQuickStyleItem::initStyleOption()
     } else {
         m_styleoption->palette.setCurrentColorGroup(QPalette::Disabled);
     }
-    if (m_active)
+
+    if (m_active) {
         m_styleoption->state |= QStyle::State_Active;
-    else
+    } else {
         m_styleoption->palette.setCurrentColorGroup(QPalette::Inactive);
+    }
+
     if (m_sunken)
         m_styleoption->state |= QStyle::State_Sunken;
     if (m_raised)
@@ -904,9 +930,17 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
     case RadioButton:
         size =  KQuickStyleItem::style()->sizeFromContents(QStyle::CT_RadioButton, m_styleoption, QSize(width,height));
         break;
-    case CheckBox:
-        size =  KQuickStyleItem::style()->sizeFromContents(QStyle::CT_CheckBox, m_styleoption, QSize(width,height));
+    case CheckBox: {
+        QStyleOptionButton *btn = qstyleoption_cast<QStyleOptionButton*>(m_styleoption);
+        QSize contentSize = btn->fontMetrics.size(Qt::TextShowMnemonic, btn->text);
+        QSize iconSize;
+        if (!btn->icon.isNull()) {
+            contentSize.setWidth(contentSize.width() + btn->iconSize.width());
+            contentSize.setHeight(std::max(contentSize.height(), btn->iconSize.height()));
+        }
+        size = KQuickStyleItem::style()->sizeFromContents(QStyle::CT_CheckBox, m_styleoption, contentSize);
         break;
+    }
     case ToolBar:
         size = QSize(200, styleName().contains(QLatin1String("windows")) ? 30 : 42);
         break;
@@ -916,8 +950,10 @@ QSize KQuickStyleItem::sizeFromContents(int width, int height)
         int h = 0;
         if (btn->toolButtonStyle != Qt::ToolButtonTextOnly) {
             QSize icon = btn->iconSize;
-            w = icon.width();
-            h = icon.height();
+            if (btn->toolButtonStyle == Qt::ToolButtonIconOnly || !btn->icon.isNull()) {
+                w = icon.width();
+                h = icon.height();
+            }
         }
         if (btn->toolButtonStyle != Qt::ToolButtonIconOnly) {
             QSize textSize = btn->fontMetrics.size(Qt::TextShowMnemonic, btn->text);
